/*============================================================================
  File:     Viewing Statistics Information.sql

  Summary:  See statistics get autocreated based on query scan. View sysindexes,
			use stats_date, use dbcc show_statistics and use dbcc show_statistics
			to produce multiple tabular data sets.
  
  Date:     June 2005

  SQL Server Version: 9.00.1116.08 (April CTP)
------------------------------------------------------------------------------
  Copyright (C) 2005 Kimberly L. Tripp, SYSolutions, Inc.
  All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  This script is intended only as a supplement to demos and lectures
  given by Kimberly L. Tripp.  
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

USE CREDIT
go

--DROP INDEX Member.MemberName
--DROP STATISTICS Member.[_WA_Sys_00000003_0CBAE877]
go

sp_helpindex Member
exec sp_helpstats Member
go

CREATE INDEX MemberName ON Member(Lastname, FirstName, MiddleInitial)
go

DBCC SHOW_STATISTICS('Member', 'MemberName')
go

sp_helpstats Member
go

SET STATISTICS IO ON
-- Turn Graphical Showplan ON (Ctrl+K)

SELECT * 
FROM dbo.Member AS m
where m.FirstName LIKE 'Kim%'
go

sp_helpstats Member
go

SELECT * FROM sys.indexes
go

-- To see all of your indexes and the last time the stats were updated
SELECT 
	object_name(si.[object_id]) 	AS [TableName]
	, CASE 
		WHEN si.[index_id] = 0 then 'Heap'
		WHEN si.[index_id] = 1 then 'CL'
		WHEN si.[index_id] BETWEEN 2 AND 250 THEN 'NC ' + RIGHT('00' + convert(varchar, si.[index_id]), 3)
		ELSE ''
	  END 							AS [IndexType]
	, si.[name] 					AS [IndexName]
	, si.[index_id]					AS [IndexID]
	, CASE
		WHEN si.[index_id] BETWEEN 1 AND 250 AND STATS_DATE (si.[object_id], si.[index_id]) < DATEADD(m, -1, getdate()) 
			THEN '!! More than a month OLD !!'
		WHEN si.[index_id] BETWEEN 1 AND 250 AND STATS_DATE (si.[object_id], si.[index_id]) < DATEADD(wk, -1, getdate()) 
			THEN '! Within the past month !'
		WHEN si.[index_id] BETWEEN 1 AND 250 THEN 'Stats recent'
		ELSE ''
	  END 							AS [Warning]
	, STATS_DATE (si.[object_id], si.[index_id]) 	AS [Last Stats Update]
FROM sys.indexes AS si
WHERE OBJECTPROPERTY(si.[object_id], 'IsUserTable') = 1
ORDER BY [TableName], si.[index_id]
go

-- Quick query w/subquery to see stats_date
SELECT stats_date(object_id('member'), 
	(SELECT index_id 
	FROM sys.indexes AS si
	WHERE object_name(si.[object_id]) = 'Member'
		AND name = 'MemberName'))
go

-- Seeing each tabular set from DBCC SHOW_STATISTICS 

DBCC SHOW_STATISTICS('Member', 'MemberName')
WITH STAT_HEADER 
go

DBCC SHOW_STATISTICS('Member', 'MemberName')
WITH DENSITY_VECTOR
go

DBCC SHOW_STATISTICS('Member', 'MemberName')
WITH HISTOGRAM
go

-- How do you use these programmatically
-- First, create a table into which this information will be inserted
-- Second, use dynamic string execution with an insert...exec to populate

CREATE TABLE MemberNameHistogram
(
	RANGE_HI_KEY			nvarchar(900),
	RANGE_ROWS				bigint,
	EQ_ROWS					bigint,
	DISTINCT_RANGE_ROWS		bigint,
	AVG_RANGE_ROWS			bigint,
)
go

INSERT MemberNameHistogram
EXEC ('DBCC SHOW_STATISTICS(''Member'', ''MemberName'') WITH HISTOGRAM')
go

SELECT * FROM MemberNameHistogram
go

SELECT 
	object_name(si.[object_id]) 	AS [TableName]
	, CASE 
		WHEN si.[stats_id] = 0 then 'Heap'
		WHEN si.[stats_id] = 1 then 'CL'
		WHEN INDEXPROPERTY ( si.[object_id], si.[name], 'IsAutoStatistics') = 1 THEN 'Stats-Auto'
		WHEN INDEXPROPERTY ( si.[object_id], si.[name], 'IsHypothetical') = 1 THEN 'Stats-HIND'
		WHEN INDEXPROPERTY ( si.[object_id], si.[name], 'IsStatistics') = 1 THEN 'Stats-User'
		WHEN si.[stats_id] BETWEEN 2 AND 250 THEN 'NC ' + RIGHT('00' + convert(varchar, si.[stats_id]), 3)
		ELSE 'Text/Image'
	  END 							AS [IndexType]
	, si.[name] 					AS [IndexName]
	, si.[stats_id]					AS [IndexID]
	, CASE
		WHEN si.[stats_id] BETWEEN 1 AND 250 AND STATS_DATE (si.[object_id], si.[stats_id]) < DATEADD(m, -1, getdate()) 
			THEN '!! More than a month OLD !!'
		WHEN si.[stats_id] BETWEEN 1 AND 250 AND STATS_DATE (si.[object_id], si.[stats_id]) < DATEADD(wk, -1, getdate()) 
			THEN '! Within the past month !'
		WHEN si.[stats_id] BETWEEN 1 AND 250 THEN 'Stats recent'
		ELSE ''
	  END 							AS [Warning]
	, STATS_DATE (si.[object_id], si.[stats_id]) 	AS [Last Stats Update]
FROM sys.stats AS si
WHERE OBJECTPROPERTY(si.[object_id], 'IsUserTable') = 1
--	AND (INDEXPROPERTY ( si.[object_id], si.[name], 'IsAutoStatistics') = 1 
--			OR INDEXPROPERTY ( si.[object_id], si.[name], 'IsHypothetical') = 1 
--			OR INDEXPROPERTY ( si.[object_id], si.[name], 'IsStatistics') = 1)
ORDER BY [Last Stats Update] DESC
go

sp_createstats 'indexonly', 'fullscan'
go

sp_helpstats member
go
-- New stats are listed as User-Stats
